/*
    This file is part of the zBoy project
    Copyright (C) Mateusz Viste 2010, 2011, 2012

    Provides GetSaveDir() - feeding a variable called
    SaveDir, that contains the directory in which zBoy
    has to store its various save files.
    If the directory doesn't exist, it will be created.
*/

#ifndef __zboy4linux__
  #ifndef __zboy4windows__
    #error "You have to #define either __zboy4linux__ or __zboy4windows__"
  #endif
#endif

#include <stdio.h>  /* sprintf() */
#include <stdlib.h> /* getenv() */

#ifdef __zboy4linux__
  #include <sys/stat.h>
#endif

#ifdef __zboy4windows__
  #include <io.h>
#endif


void CreateDirectoryIfNotExists(char *dir) {
  #ifdef __zboy4linux__
    mkdir(dir, S_IRWXU);
  #endif
  #ifdef __zboy4windows__
    _mkdir(dir);
  #endif
}


void GetSaveDir(char *SaveDir) {
  char *userdir;
  #ifdef __zboy4linux__
    userdir = getenv("HOME");
    if (userdir != NULL) {
        sprintf(SaveDir, "%s%s", userdir, "/.zboy/");
      } else {
        SaveDir[0] = 0;
    }
  #endif
  #ifdef __zboy4windows__
    userdir = getenv("USERPROFILE");
    if (userdir != NULL) {
        sprintf(SaveDir, "%s%s", userdir, "\\zboy_sav\\");
      } else {
        SaveDir[0] = 0;
    }
  #endif
  if (SaveDir[0] != 0) CreateDirectoryIfNotExists(SaveDir);
}
