/*
   LoadPalette()  -  Loads a color palette from an external file.

   This file is part of the zBoy project.
   Copyright (C) Mateusz Viste 2010, 2011, 2012
*/

#include <string.h> /* strcasecmp() */
#include <stdint.h>

int LoadPalette(char *palfile, unsigned int *ScreenPalette32) {
  ScreenPalette32[254] = 0x000000; /* zBoy text foreground */
  ScreenPalette32[255] = 0xFFFFFF; /* zBoy text background */
  if (strcasecmp(palfile, "gbmicro") == 0) {
    ScreenPalette32[0] = 0xFFFFFF; /* background */
    ScreenPalette32[1] = 0xAAAAAA;
    ScreenPalette32[2] = 0x555555;
    ScreenPalette32[3] = 0x000000;
  } else if (strcasecmp(palfile, "peasoup") == 0) {
    ScreenPalette32[0] = 0xD5EA94;
    ScreenPalette32[1] = 0xACC641;
    ScreenPalette32[2] = 0x527D39;
    ScreenPalette32[3] = 0x204431;
  } else {
    return(1);
  }
  /* synch colors for window, OBJ0 and OBJ1 */
  ScreenPalette32[32] = ScreenPalette32[0];
  ScreenPalette32[33] = ScreenPalette32[1];
  ScreenPalette32[34] = ScreenPalette32[2];
  ScreenPalette32[35] = ScreenPalette32[3];
  ScreenPalette32[64] = ScreenPalette32[0];
  ScreenPalette32[65] = ScreenPalette32[1];
  ScreenPalette32[66] = ScreenPalette32[2];
  ScreenPalette32[67] = ScreenPalette32[3];
  ScreenPalette32[128] = ScreenPalette32[0];
  ScreenPalette32[129] = ScreenPalette32[1];
  ScreenPalette32[130] = ScreenPalette32[2];
  ScreenPalette32[131] = ScreenPalette32[3];
  return(0);
  /*
  FILE *FileHandler
  char TempLine[1024];
  uint8_t Rvalue, Gvalue, Bvalue, x;
  FileHandler = open(palfile, "rb");
  if (FileHandler != NULL) {  // File opened okay
      LINE INPUT #FileHandler, TempLine
      if (strcmp(TempLine, "JASC-PAL") == 0) {
          LINE INPUT #FileHandler, TempLine  // Version
          if (TRIM(TempLine) = "0100") {  // check the version
              LINE INPUT #FileHandler, TempLine  // Number of colors
              if (VAL(TempLine) = 16) {   // check the number of colors in the palette
                  for (x = 0; x < 16; x++) {
                    LINE INPUT #FileHandler, TempLine // get the color definition
                    Rvalue = VAL(LEFT(TempLine, INSTR(TempLine, " ") - 1));
                    TempLine = MID(TempLine, INSTR(TempLine, " ") + 1);
                    Gvalue = VAL(LEFT(TempLine, INSTR(TempLine, " ") - 1));
                    TempLine = MID(TempLine, INSTR(TempLine, " ") + 1);
                    Bvalue = VAL(TempLine);
                    // * Load 32BPP palette *
                    ScreenPalette32[x] = (Rvalue << 16) | (Gvalue << 8) | Bvalue;
                  }
                  fclose(FileHandler);
                } else {   // Invalid number of colors
                  sprintf(TempLine, "ERROR: THE PALETTE FILE CONTAINS AN UNSUPPORTED NUMBER OF COLORS!\n\nZBOY SUPPORTS ONLY JASC PALETTES WITH 16 COLORS.\n\nPRESS ANY KEY TO QUIT...");
                  PrintMsg(TempLine);
                  fclose(FileHandler);
                  PressAnyKey();
                  exit(0);
              }
            } else {   // Unknown palette version
              sprintf(TempLine, "ERROR: THE SPECIFIED PALETTE FILE VERSION IS NOT SUPPORTED!\n\nZBOY SUPPORTS ONLY JASC PALETTES IN VERSION 0100.\n\nPRESS ANY KEY TO QUIT...");
              PrintMsg(TempLine);
              fclose(FileHandler);
              PressAnyKey();
              exit(0);
          }
        } else {
          sprintf(TempLine, "ERROR: THE SPECIFIED PALETTE FILE IS NOT A VALID JASC PALETTE!\n\nPRESS ANY KEY TO QUIT...");
          PrintMsg(TempLine);
          fclose(FileHandler);
          PressAnyKey();
          exit(0);
      }
    } else {
      sprintf(TempLine, "ERROR: THE SPECIFIED PALETTE FILE COULD NOT BE OPENED.\n\nPRESS ANY KEY TO QUIT...");
      PrintMsg(TempLine);
      PressAnyKey();
      exit(0);
  } */
}


void CheckForColorizedPalette(uint32_t romcrc, unsigned int *ScreenPalette32) {
  switch (romcrc) {
    case 0x90776841:  /* Super Marioland v1.0 */
    case 0x2C27EC70:  /* Super Marioland v1.1 */
      ScreenPalette32[0] = 0xE3E2CC; /* background */
      ScreenPalette32[1] = 0xC1AE6B; /* background */
      ScreenPalette32[2] = 0x6D3624; /* background */
      ScreenPalette32[3] = 0x000000; /* background */
      ScreenPalette32[32] = 0xFFFFC0; /* window */
      ScreenPalette32[33] = 0xFF5050; /* window */
      ScreenPalette32[34] = 0x555555; /* window */
      ScreenPalette32[35] = 0x101010; /* window */
      ScreenPalette32[64] = 0x00F0F0; /* OBJ0 */
      ScreenPalette32[65] = 0xDB7636; /* OBJ0 - mario skin */
      ScreenPalette32[66] = 0xA74925; /* OBJ0 - mario pants */
      ScreenPalette32[67] = 0x101010; /* OBJ0 */
      ScreenPalette32[128] = 0x0000FF; /* OBJ1 */
      ScreenPalette32[129] = 0x0000AA; /* OBJ1 */
      ScreenPalette32[130] = 0x000055; /* OBJ1 */
      ScreenPalette32[131] = 0x0000F0; /* OBJ1 */
      break;
    case 0x63F9407D:  /* Tetris v1.0 */
    case 0x46DF91AD:  /* Tetris v1.1 */
      ScreenPalette32[0] = 0xE3E2CC; /* background */
      ScreenPalette32[1] = 0x000000; /* background */
      ScreenPalette32[2] = 0xA24420; /* background */
      ScreenPalette32[3] = 0x001000; /* background */
      ScreenPalette32[32] = 0xFFFFC0; /* window */
      ScreenPalette32[33] = 0xFF5050; /* window */
      ScreenPalette32[34] = 0x555555; /* window */
      ScreenPalette32[35] = 0x101010; /* window */
      ScreenPalette32[64] = 0xE3E2CC; /* OBJ0 */
      ScreenPalette32[65] = 0x2549A7; /* OBJ0 */
      ScreenPalette32[66] = 0xA74925; /* OBJ0 */
      ScreenPalette32[67] = 0x001000; /* OBJ0 */
      ScreenPalette32[128] = 0x00FF00; /* OBJ1 */
      ScreenPalette32[129] = 0x00FF00; /* OBJ1 */
      ScreenPalette32[130] = 0x00FF00; /* OBJ1 */
      ScreenPalette32[131] = 0x00FF00; /* OBJ1 */
      break;
    case 0x6C742478:  /* Dig Dug */
      ScreenPalette32[0] = 0xDE6800; /* background - highest level ground */
      ScreenPalette32[1] = 0xA0A0B0; /* background - tunnels and sky */
      ScreenPalette32[2] = 0xB82100; /* background - middle level ground */
      ScreenPalette32[3] = 0x950000; /* background - deepest level ground */
      ScreenPalette32[32] = 0x000000; /* window - status bar background */
      ScreenPalette32[33] = 0x0000FF; /* window - dig dug eyes in status bar */
      ScreenPalette32[34] = 0x555555; /* window */
      ScreenPalette32[35] = 0xCECECE; /* window - status bar text and ddug guy */
      ScreenPalette32[64] = 0xE3E2CC; /* OBJ0 - Dig dug body */
      ScreenPalette32[65] = ScreenPalette32[1]; /* OBJ0 - dig dug eyes (must be same as tunnels) */
      ScreenPalette32[66] = 0xA74925; /* OBJ0 */
      ScreenPalette32[67] = 0x000000; /* OBJ0 - dig dug weapon and aura */
      ScreenPalette32[128] = 0xF0E000; /* OBJ1 - vilains eyes and rocks contours when falling */
      ScreenPalette32[129] = 0x00FF00; /* OBJ1 */
      ScreenPalette32[130] = 0xE83020; /* OBJ1 - Dragons bodies */
      ScreenPalette32[131] = 0x101010; /* OBJ1 - vilains contours */
      break;
    case 0x79804305:  /* Pinball Revenge of the Gator */
      ScreenPalette32[0] = 0xF0F0F0; /* background */
      ScreenPalette32[1] = 0x00AA00; /* background */
      ScreenPalette32[2] = 0x005500; /* background */
      ScreenPalette32[3] = 0x000000; /* background */
      ScreenPalette32[32] = 0x00FFFF; /* window */
      ScreenPalette32[33] = 0x00FFFF; /* window */
      ScreenPalette32[34] = 0x00FFFF; /* window */
      ScreenPalette32[35] = 0x00FFFF; /* window */
      ScreenPalette32[64] = 0x00FF00; /* OBJ0 most sprites use OBJ0 */
      ScreenPalette32[65] = 0x00AA00; /* OBJ0 */
      ScreenPalette32[66] = 0x005500; /* OBJ0 */
      ScreenPalette32[67] = 0x002000; /* OBJ0 */
      ScreenPalette32[128] = 0xFF4040; /* OBJ1 few elementes are OBJ1 */
      ScreenPalette32[129] = 0xAA3030; /* OBJ1 */
      ScreenPalette32[130] = 0x552020; /* OBJ1  */
      ScreenPalette32[131] = 0x100000; /* OBJ1  */
      break;
    case 0xC1F88833:  /* Asteroids */
      ScreenPalette32[0] = 0xc7caff; /* background */
      ScreenPalette32[1] = 0x686fff; /* background */
      ScreenPalette32[2] = 0x000cff; /* background */
      ScreenPalette32[3] = 0x000020; /* background */
      ScreenPalette32[32] = 0x00FFFF; /* window */
      ScreenPalette32[33] = 0x00FFFF; /* window */
      ScreenPalette32[34] = 0x00FFFF; /* window */
      ScreenPalette32[35] = 0x00FFFF; /* window */
      ScreenPalette32[64] = 0xc15200; /* OBJ0 all sprites use OBJ0 */
      ScreenPalette32[65] = 0x7a3b0d; /* OBJ0 */
      ScreenPalette32[66] = 0x9d4c11; /* OBJ0 */
      ScreenPalette32[67] = 0x000000; /* OBJ0 */
      ScreenPalette32[128] = 0xFF00FF; /* OBJ1 */
      ScreenPalette32[129] = 0xAA00AA; /* OBJ1 */
      ScreenPalette32[130] = 0x550055; /* OBJ1 */
      ScreenPalette32[131] = 0x100010; /* OBJ1 */
      break;
    case 0x0509069C:  /* Pac-Man (Namco) */
    case 0xB681E243:  /* Pac-Man (Namco & Hometek) */
      ScreenPalette32[0] = 0xf0f0f0; /* background - maze background */
      ScreenPalette32[1] = 0xE00000; /* background - maze borders when flashing and PacMan text on title screen */
      ScreenPalette32[2] = 0x5757ff; /* background - maze borders */
      ScreenPalette32[3] = 0x000080; /* background - dots and 'ready' text */
      ScreenPalette32[32] = 0xF1F335; /* window - body of little pacman lifes */
      ScreenPalette32[33] = 0xab6434; /* window - panel background */
      ScreenPalette32[34] = 0xff8303; /* window - fruits color */
      ScreenPalette32[35] = 0x202020; /* window - panel text and little pacman lifes aura */
      ScreenPalette32[64] = 0xFF00FF; /* OBJ0 all sprites use OBJ0 */
      ScreenPalette32[65] = 0xF1F335; /* OBJ0 - Pacman body and ghost's eyes */
      ScreenPalette32[66] = 0xF05703; /* OBJ0 - ghosts bodies*/
      ScreenPalette32[67] = 0x101010; /* OBJ0 - pacman aura and points text */
      ScreenPalette32[128] = 0xFF00FF; /* OBJ1 - ghosts switch to OBJ1 when being chased */
      ScreenPalette32[129] = 0xD0D0D0; /* OBJ1 - ghost's mouth when chased */
      ScreenPalette32[130] = 0x00FF00; /* OBJ1 */
      ScreenPalette32[131] = 0x2A2AF3; /* OBJ1 - ghost's body when chased */
      break;
    case 0xB76C769B: /* Tetris attack */
      ScreenPalette32[0] = 0xf0f0f0; /* background */
      ScreenPalette32[1] = 0x34c430; /* background - circles (interior) */
      ScreenPalette32[2] = 0xdb5228; /* background - hearths (interior) and stars (exterior) */
      ScreenPalette32[3] = 0x101010; /* background - aura of star elements */
      ScreenPalette32[32] = 0xF0F0F0; /* window - background */
      ScreenPalette32[33] = 0x0da010; /* window */
      ScreenPalette32[34] = 0x69d46b; /* window */
      ScreenPalette32[35] = 0x202020; /* window - panel text */
      ScreenPalette32[64] = 0x00FFFF; /* OBJ0 all sprites use OBJ0 */
      ScreenPalette32[65] = 0x5d7cc9; /* OBJ0 - taints on eggs */
      ScreenPalette32[66] = 0x303030; /* OBJ0 - aura on eggs */
      ScreenPalette32[67] = 0x000000; /* OBJ0 */
      ScreenPalette32[128] = 0x9a5628; /* OBJ1 - internal content of selectors */
      ScreenPalette32[129] = 0x303030; /* OBJ1 - selectors internal elements and text */
      ScreenPalette32[130] = 0xFF00FF; /* OBJ1 */
      ScreenPalette32[131] = 0x000000; /* OBJ1 - aura of selectors */
      break;
  }
}
