/*
   Saves the content of ScreenBuffer to OutFile as a 160x144 PCX graphic file.
   This file is part of the zBoy emulator project.
   Copyright (C) Mateusz Viste 2010,2011,2012
*/

#include <stdint.h>
#include <stdio.h>

void SavePcxFile(char *OutFile) {
  int x, y;
  FILE *PcxFileHandler;
  uint8_t RleCounter = 0, OldPixel;
  uint8_t PcxDataHeader[128] = {10, 5,   1, 8, 0, 0, 0, 0, 159, 0, 143, 0, 10, 0,10, 0,
                                 0, 0,   0, 0, 0, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0,
                                 0, 0,   0, 0, 0, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0,
                                 0, 0,   0, 0, 0, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0,
                                 0, 1, 160, 0, 1, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0,
                                 0, 0,   0, 0, 0, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0,
                                 0, 0,   0, 0, 0, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0,
                                 0, 0,   0, 0, 0, 0, 0, 0,   0, 0,   0, 0,  0, 0, 0, 0};
  /*unsigned char PaletteTrailer[756]; */
  PcxFileHandler = fopen(OutFile, "wb");
  if (PcxFileHandler != NULL) {
    /* Write PCX header */ /*
    PRINT #PcxFileHandler, CHR(10);                     ' ZSoft file ID
    PRINT #PcxFileHandler, CHR(5);                      ' PCX version (0, 1, 2, 3, 4 or 5)
    PRINT #PcxFileHandler, CHR(1);                      ' encoding (1 = RLE)
    PRINT #PcxFileHandler, CHR(8);                      ' Bits depth (8 = 256 colors)
    PRINT #PcxFileHandler, CHR(0, 0);                   ' xmin
    PRINT #PcxFileHandler, CHR(0, 0);                   ' ymin
    PRINT #PcxFileHandler, CHR(159, 0);                 ' xmax
    PRINT #PcxFileHandler, CHR(143, 0);                 ' ymax
    PRINT #PcxFileHandler, CHR(10, 0);                  ' Horizontal DPI (2 bytes)
    PRINT #PcxFileHandler, CHR(10, 0);                  ' Vertical DPI (2 bytes)
    PRINT #PcxFileHandler, STRING(48, 0);               ' Palette for 16 colors (48 bytes)
    PRINT #PcxFileHandler, CHR(0);                      ' reserved
    PRINT #PcxFileHandler, CHR(1);                      ' number of color planes
    PRINT #PcxFileHandler, CHR(160, 0);                 ' nb of bytes per line (in 8bit mode = image width)
    PRINT #PcxFileHandler, CHR(1, 0);                   ' PaletteType (1=color/bw  2=grayscale)
    PRINT #PcxFileHandler, CHR(0, 0);                   ' HScrSize (optional)
    PRINT #PcxFileHandler, CHR(0, 0);                   ' VScrSize (optional)
    PRINT #PcxFileHandler, STRING(54, 0);               ' Filler */
    for (x = 0; x < 128; x++) fputc(PcxDataHeader[x], PcxFileHandler);
    /* Write PCX image data */
    for (y = 0; y < 144; y++) {
      OldPixel = ScreenBuffer[0][y];
      RleCounter = 1;
      for (x = 1; x < 160; x++) {
        if ((ScreenBuffer[x][y] == OldPixel) && (RleCounter < 63)) {
            RleCounter++;
          } else {
            if ((RleCounter > 1) || (OldPixel > 191)) {
              RleCounter |= 192;
              fputc(RleCounter, PcxFileHandler);
            }
            fputc(OldPixel, PcxFileHandler);
            RleCounter = 1;
            OldPixel = ScreenBuffer[x][y];
        }
      }
      if ((RleCounter > 1) || (OldPixel > 191)) {
        RleCounter |= 192;
        fputc(RleCounter, PcxFileHandler);
      }
      fputc(OldPixel, PcxFileHandler);
    }
    /* Write RGB palette (768 bytes) */
    fputc(12, PcxFileHandler);                        /* Palette byte ID (12) */
    for (x = 0; x < 256; x++) {
      OldPixel = ((ScreenPalette32[x] >> 16) & 255);  /* Get Red value */
      fputc(OldPixel, PcxFileHandler);                /* Write Red value */
      OldPixel = ((ScreenPalette32[x] >> 8) & 255);   /* Get Green value */
      fputc(OldPixel, PcxFileHandler);                /* Write Green value */
      OldPixel = ((ScreenPalette32[x]) & 255);        /* Get Blue value */
      fputc(OldPixel, PcxFileHandler);                /* Write Blue value */
    }
    fclose(PcxFileHandler);
  }
}
