/*
   zBoy GTK GUI
   Copyright (C) Mateusz Viste 2010-2012

  zboygui returns:
   0 - let's play
   1 - wants to exit
*/

#include <gtk/gtk.h>
#include <string.h>  /* strlen() */
#include "zboystructs.h"
#include "loadrom.h"
#include "config.h"           /* zBoy_loadconf() & zBoy_saveconf() */


/* Declare the prototype for zboyversion() and zboydate() */
char *zboyversion();
char *zboydate();

/* Prototype declaration for int2keystring (from joypad.c)
   returns a pointer to a string, or NULL if keycode is invalid */
char *int2keystring(int keycode);


/* Declare a structure that will be used to pass information between GTK routines */
struct zboygtkgui_infos {
  GtkWidget *parentwindow;
  GtkWidget *curwindow;
  char *romfilename;
  int playclicked;
  int exitclicked;
  int newromtoload;
  struct RomInformations *rominfo;
  struct zboyparamstype *curconf;
  struct zboyparamstype *workconf;
};

/* this struct is used by the videosettings screen to exchange widget handlers with callback functions */
struct zboygui_video {
  GtkWidget *radio_alg_dummy;
  GtkWidget *radio_alg_none;
  GtkWidget *radio_alg_basic;
  GtkWidget *radio_alg_2x;
  GtkWidget *radio_alg_3x;
  GtkWidget *radio_alg_eagle;
  GtkWidget *radio_scal1;
  GtkWidget *radio_scal2;
  GtkWidget *radio_scal3;
  GtkWidget *radio_scal4;
  GtkWidget *radio_scal5;
  GtkWidget *radio_scal6;
  GtkWidget *radio_scal7;
  GtkWidget *radio_scal8;
  GtkWidget *spin_refreshlimit;
  GtkWidget *combo_palette;
  GtkWidget *chbox_colorization;
  struct zboygtkgui_infos *zboystruct;
};


void NotImplementedYet(GtkWidget *widget, gpointer userdata) {
  GtkWidget *dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "Not implemented yet!\nSorry.");
  gtk_dialog_run(GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);
}


void openfile(GtkWidget *widget, gpointer userdata) {
  char filename[1024] = {0};
  char *result;
  struct zboygtkgui_infos *zboygtkinfos = (struct zboygtkgui_infos *)userdata;
  GtkWidget *fileopendialog;
  fileopendialog = gtk_file_chooser_dialog_new("Open File", NULL, GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
  if (gtk_dialog_run(GTK_DIALOG(fileopendialog)) == GTK_RESPONSE_ACCEPT) {
    result = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(fileopendialog));
    if (result != NULL) {
      snprintf(filename, 1024, "%s", result);
    }
  }
  gtk_widget_destroy(fileopendialog);
  if (filename[0] != 0) {
    zboygtkinfos->newromtoload = 1;
    sprintf(zboygtkinfos->rominfo->Filename, "%s", filename);
    gtk_main_quit();
  }
}


void abortwindow(GtkWidget *parentwidget, gpointer userdata) {
  GtkWidget *destroythis = (GtkWidget *)userdata;
  gtk_widget_destroy(destroythis);
}


void aboutscreen(GtkWidget *parentwidget, gpointer userdata) {
  GtkWidget *topwindow = (GtkWidget *)userdata;
  char tempstring[1024], *pVer;
  GtkWidget *window, *vbox, *label_title, *label_content, *alignement, *alignment_button, *button_close;
  char *aboutcontent[] = {"zBoy is an opensource, cross-platform GameBOY&#xAE; classic emulator written in C. The emulator is released under the GNU GPL license.",
                          "Copyright &#xA9; Mateusz Viste ",
                          "This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.",
                          "The zBoy emulator is not, and will not be shipped with any actual GameBoy game. GameBoy games are copyrighted by companies which produced them. The author is in no way affiliated or endorsed by Nintendo or Nintendo of America. \"Nintendo\" is a registered trademark of Nintendo of America Inc."};
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "About zBoy");
  gtk_window_set_default_size(GTK_WINDOW(window), 400, 400);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE); /* don't allow to manipulate the main window when this one is displayed */
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(topwindow)); /* Don't allow to steal the focus of this window */
  alignement = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(alignement), 30, 10, 10, 10);
  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_add(GTK_CONTAINER(window), alignement);
  gtk_container_add(GTK_CONTAINER(alignement), vbox);
  label_title = gtk_label_new(NULL);
  label_content = gtk_label_new(NULL);
  pVer = zboyversion();
  snprintf(tempstring, 1024, "<span font_size=\"larger\">zBoy v%s, a GameBOY&#xAE; classic emulator</span>\n<span font_size=\"smaller\">http://www.viste-family.net/mateusz/software/zboy/</span>\n", pVer);
  gtk_label_set_markup(GTK_LABEL(label_title), tempstring);
  gtk_label_set_justify(GTK_LABEL(label_title), GTK_JUSTIFY_CENTER);
  label_content = gtk_label_new(NULL);
  snprintf(tempstring, 1024, "%s\n\n%s%s\n\n%s\n\n%s", aboutcontent[0], aboutcontent[1], zboydate(), aboutcontent[2], aboutcontent[3]);
  gtk_label_set_markup(GTK_LABEL(label_content), tempstring);
  gtk_label_set_justify(GTK_LABEL(label_content), GTK_JUSTIFY_FILL);
  gtk_label_set_line_wrap(GTK_LABEL(label_content), TRUE);
  /* Prepare the close button */
  button_close = gtk_button_new_with_label("Close");
  g_signal_connect(G_OBJECT(button_close), "clicked", G_CALLBACK(abortwindow), window);
  /* pack labels and buttons into the vbox */
  gtk_box_pack_start(GTK_BOX(vbox), label_title, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), label_content, FALSE, FALSE, 0);
  alignment_button = gtk_alignment_new(0.5, 1, 0, 0);
  gtk_container_add(GTK_CONTAINER(alignment_button), button_close);
  gtk_box_pack_end(GTK_BOX(vbox), alignment_button, FALSE, FALSE, 0);
  gtk_widget_show_all(window);
}


void PopulateWorkConfFromCurConf(struct zboygtkgui_infos *zboystruct) {
  zboystruct->workconf->fpslimit = zboystruct->curconf->fpslimit; /* fpslimit */
  zboystruct->workconf->scalingalgo = zboystruct->curconf->scalingalgo; /* GraphicScaleFactor */
  zboystruct->workconf->GraphicScaleFactor = zboystruct->curconf->GraphicScaleFactor; /* GraphicScaleFactor */
  zboystruct->workconf->ShowFPS = zboystruct->curconf->ShowFPS; /* showfps */
  zboystruct->workconf->NoScreenSaver = zboystruct->curconf->NoScreenSaver; /* noscreensaver */
  zboystruct->workconf->NoSpeedLimit = zboystruct->curconf->NoSpeedLimit; /* nospeedlimit */
  zboystruct->workconf->HiScoresMem = zboystruct->curconf->HiScoresMem; /* HiScoresMem */
  zboystruct->workconf->NoCpuIdle = zboystruct->curconf->NoCpuIdle; /* NoCpuIdle */
  zboystruct->workconf->bootlogo = zboystruct->curconf->bootlogo; /* NoCpuIdle */
  zboystruct->workconf->palette = zboystruct->curconf->palette; /* palette */
  zboystruct->workconf->colorization = zboystruct->curconf->colorization; /* colorization */
  /* key mappings */
  zboystruct->workconf->key_up = zboystruct->curconf->key_up;
  zboystruct->workconf->key_down = zboystruct->curconf->key_down;
  zboystruct->workconf->key_left = zboystruct->curconf->key_left;
  zboystruct->workconf->key_right = zboystruct->curconf->key_right;
  zboystruct->workconf->key_start = zboystruct->curconf->key_start;
  zboystruct->workconf->key_select = zboystruct->curconf->key_select;
  zboystruct->workconf->key_a = zboystruct->curconf->key_a;
  zboystruct->workconf->key_b = zboystruct->curconf->key_b;
  zboystruct->workconf->key_turboa = zboystruct->curconf->key_turboa;
  zboystruct->workconf->key_turbob = zboystruct->curconf->key_turbob;
  zboystruct->workconf->key_bckg = zboystruct->curconf->key_bckg;
  zboystruct->workconf->key_sprt = zboystruct->curconf->key_sprt;
  zboystruct->workconf->key_wind = zboystruct->curconf->key_wind;
  zboystruct->workconf->key_save = zboystruct->curconf->key_save;
  zboystruct->workconf->key_load = zboystruct->curconf->key_load;
  zboystruct->workconf->key_asht = zboystruct->curconf->key_asht;
  zboystruct->workconf->key_shot = zboystruct->curconf->key_shot;
  zboystruct->workconf->key_rset = zboystruct->curconf->key_rset;
  zboystruct->workconf->key_quit = zboystruct->curconf->key_quit;
}


void saveguiconfig(GtkWidget *widget, gpointer userdata) {
  struct zboygtkgui_infos *zboystruct = (struct zboygtkgui_infos *)userdata;
  /* Copy workconfig into curconfig */
  zboystruct->curconf->ShowFPS = zboystruct->workconf->ShowFPS; /* showfps */
  zboystruct->curconf->NoScreenSaver = zboystruct->workconf->NoScreenSaver; /* noscreensaver */
  zboystruct->curconf->NoSpeedLimit = zboystruct->workconf->NoSpeedLimit; /* nospeedlimit */
  zboystruct->curconf->HiScoresMem = zboystruct->workconf->HiScoresMem; /* HiScoresMem */
  zboystruct->curconf->NoCpuIdle = zboystruct->workconf->NoCpuIdle; /* NoCpuIdle */
  zboystruct->curconf->bootlogo = zboystruct->workconf->bootlogo; /* NoCpuIdle */
  zboystruct->curconf->scalingalgo = zboystruct->workconf->scalingalgo; /* scalingalgo */
  zboystruct->curconf->GraphicScaleFactor = zboystruct->workconf->GraphicScaleFactor; /* GraphicScaleFactor */
  zboystruct->curconf->fpslimit = zboystruct->workconf->fpslimit; /* fpslimit */
  zboystruct->curconf->palette = zboystruct->workconf->palette; /* palette */
  zboystruct->curconf->colorization = zboystruct->workconf->colorization; /* colorization */
  /* key mappings */
  zboystruct->curconf->key_up = zboystruct->workconf->key_up;
  zboystruct->curconf->key_down = zboystruct->workconf->key_down;
  zboystruct->curconf->key_left = zboystruct->workconf->key_left;
  zboystruct->curconf->key_right = zboystruct->workconf->key_right;
  zboystruct->curconf->key_start = zboystruct->workconf->key_start;
  zboystruct->curconf->key_select = zboystruct->workconf->key_select;
  zboystruct->curconf->key_a = zboystruct->workconf->key_a;
  zboystruct->curconf->key_b = zboystruct->workconf->key_b;
  zboystruct->curconf->key_turboa = zboystruct->workconf->key_turboa;
  zboystruct->curconf->key_turbob = zboystruct->workconf->key_turbob;
  zboystruct->curconf->key_bckg = zboystruct->workconf->key_bckg;
  zboystruct->curconf->key_sprt = zboystruct->workconf->key_sprt;
  zboystruct->curconf->key_wind = zboystruct->workconf->key_wind;
  zboystruct->curconf->key_save = zboystruct->workconf->key_save;
  zboystruct->curconf->key_load = zboystruct->workconf->key_load;
  zboystruct->curconf->key_asht = zboystruct->workconf->key_asht;
  zboystruct->curconf->key_shot = zboystruct->workconf->key_shot;
  zboystruct->curconf->key_rset = zboystruct->workconf->key_rset;
  zboystruct->curconf->key_quit = zboystruct->workconf->key_quit;
  /* save configuration to a file on disk */
  zBoy_saveconfig(zboystruct->curconf);
  /* destroy the configuration window */
  gtk_widget_destroy(zboystruct->curwindow);
}


void togglebutton_uint8(GtkWidget *widget, gpointer userdata) {
  uint8_t *switchbutton = (uint8_t *)userdata;
  if (*switchbutton == 0) *switchbutton = 1; else *switchbutton = 0;
}


void generalsettingsscreen(GtkWidget *widget, gpointer userdata) {
  GtkWidget *window, *vbox, *hbox_buttons, *label_title, *alignement, *alignment_buttons;
  struct zboygtkgui_infos *zboystruct = (struct zboygtkgui_infos *)userdata;
  GtkWidget *topwindow = zboystruct->parentwindow;
  GtkWidget *button_ok, *button_cancel;
  GtkWidget *chbox_bootlogo, *chbox_hiscoresmem, *chbox_showfps, *chbox_nossaver, *chbox_nocpuidle, *chbox_nospeedlimit;
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "zBoy general settings");
  gtk_window_set_default_size(GTK_WINDOW(window), 300, 150);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  alignement = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(alignement), 10, 10, 10, 10);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE); /* don't allow to manipulate the main window when this one is displayed */
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(topwindow)); /* Don't allow to steal the focus of this window */
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_container_add(GTK_CONTAINER(window), alignement);
  gtk_container_add(GTK_CONTAINER(alignement), vbox);
  label_title = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label_title), "General settings\n");
  gtk_label_set_justify(GTK_LABEL(label_title), GTK_JUSTIFY_CENTER);

  /* Prepare checkboxes */
  chbox_bootlogo = gtk_check_button_new_with_label("Display boot logo");
  chbox_hiscoresmem = gtk_check_button_new_with_label("Enable hi-scores memory");
  chbox_showfps = gtk_check_button_new_with_label("Show FPS");
  chbox_nossaver = gtk_check_button_new_with_label("Inhibit screensavers");
  chbox_nocpuidle = gtk_check_button_new_with_label("Disable CPU idling (not recommended!)");
  chbox_nospeedlimit = gtk_check_button_new_with_label("Unlimited speed");

  /* Set checkboxes accordingly to current params, and synch workparams with curparams */
  PopulateWorkConfFromCurConf(zboystruct);
  if (zboystruct->curconf->ShowFPS != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_showfps), TRUE);
  if (zboystruct->curconf->NoScreenSaver != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_nossaver), TRUE);
  if (zboystruct->curconf->NoSpeedLimit != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_nospeedlimit), TRUE);
  if (zboystruct->curconf->HiScoresMem != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_hiscoresmem), TRUE);
  if (zboystruct->curconf->NoCpuIdle != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_nocpuidle), TRUE);
  if (zboystruct->curconf->bootlogo != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_bootlogo), TRUE);

  /* wire buttons */
  g_signal_connect(chbox_bootlogo, "toggled", G_CALLBACK(togglebutton_uint8), &zboystruct->workconf->bootlogo);
  g_signal_connect(chbox_hiscoresmem, "toggled", G_CALLBACK(togglebutton_uint8), &zboystruct->workconf->HiScoresMem);
  g_signal_connect(chbox_showfps, "toggled", G_CALLBACK(togglebutton_uint8), &zboystruct->workconf->ShowFPS);
  g_signal_connect(chbox_nossaver, "toggled", G_CALLBACK(togglebutton_uint8), &zboystruct->workconf->NoScreenSaver);
  g_signal_connect(chbox_nocpuidle, "toggled", G_CALLBACK(togglebutton_uint8), &zboystruct->workconf->NoCpuIdle);
  g_signal_connect(chbox_nospeedlimit, "toggled", G_CALLBACK(togglebutton_uint8), &zboystruct->workconf->NoSpeedLimit);

  /* pack the vbox with stuff */
  gtk_box_pack_start(GTK_BOX(vbox), label_title, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_bootlogo, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_hiscoresmem, FALSE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_showfps, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_nossaver, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_nocpuidle, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_nospeedlimit, FALSE, FALSE, 0);

  /* finally pack in OK / CANCEL buttons */
  hbox_buttons = gtk_hbox_new(FALSE, 10);
  alignment_buttons = gtk_alignment_new(1, 1, 0, 0);

  gtk_container_add(GTK_CONTAINER(alignment_buttons), hbox_buttons);

  button_ok = gtk_button_new_with_label("OK");
  button_cancel = gtk_button_new_with_label("Cancel");
  gtk_box_pack_start(GTK_BOX(hbox_buttons), button_cancel, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_buttons), button_ok, FALSE, FALSE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), alignment_buttons, FALSE, FALSE, 0);

  zboystruct->curwindow = window;
  g_signal_connect(G_OBJECT(button_ok), "clicked", G_CALLBACK(saveguiconfig), zboystruct);
  g_signal_connect(G_OBJECT(button_cancel), "clicked", G_CALLBACK(abortwindow), window);

  gtk_widget_show_all(window);
}


void radio_select_algo_none(GtkWidget *widget, gpointer userdata) {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) {
    struct zboygui_video *widgets = (struct zboygui_video *)userdata;
    widgets->zboystruct->workconf->scalingalgo = REFRESHSCREEN_NOSCALE;
    gtk_widget_set_sensitive(widgets->radio_scal1, TRUE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal1), TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal2, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal3, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal4, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal5, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal6, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal7, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal8, FALSE);
  }
}

void radio_select_algo_basic(GtkWidget *widget, gpointer userdata) {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) {
    struct zboygui_video *widgets = (struct zboygui_video *)userdata;
    widgets->zboystruct->workconf->scalingalgo = REFRESHSCREEN_BASICSCALE;
    gtk_widget_set_sensitive(widgets->radio_scal1, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal2, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal3, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal4, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal5, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal6, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal7, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal8, TRUE);
  }
}

void radio_select_algo_2x(GtkWidget *widget, gpointer userdata) {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) {
    struct zboygui_video *widgets = (struct zboygui_video *)userdata;
    widgets->zboystruct->workconf->scalingalgo = REFRESHSCREEN_2X;
    gtk_widget_set_sensitive(widgets->radio_scal2, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal4, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal6, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal8, TRUE);
    switch (widgets->zboystruct->workconf->GraphicScaleFactor) {
      case 2:
      case 4:
      case 6:
      case 8: /* these factors are okay, leave them as is */
        break;
      case 5:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal4), TRUE);
        break;
      case 7:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal6), TRUE);
        break;
      default:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal2), TRUE);
        break;
    }
    gtk_widget_set_sensitive(widgets->radio_scal1, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal3, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal5, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal7, FALSE);
  }
}

void radio_select_algo_3x(GtkWidget *widget, gpointer userdata) {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) {
    struct zboygui_video *widgets = (struct zboygui_video *)userdata;
    widgets->zboystruct->workconf->scalingalgo = REFRESHSCREEN_3X;
    gtk_widget_set_sensitive(widgets->radio_scal3, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal6, TRUE);
    switch (widgets->zboystruct->workconf->GraphicScaleFactor) {
      case 3:
      case 6: /* these factors are okay, leave them as is */
        break;
      case 7:
      case 8:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal6), TRUE);
        break;
      default:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal3), TRUE);
        break;
    }
    gtk_widget_set_sensitive(widgets->radio_scal1, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal2, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal4, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal5, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal7, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal8, FALSE);
  }
}

void radio_select_algo_eagle(GtkWidget *widget, gpointer userdata) {
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) {
    struct zboygui_video *widgets = (struct zboygui_video *)userdata;
    widgets->zboystruct->workconf->scalingalgo = REFRESHSCREEN_EAGLE;
    gtk_widget_set_sensitive(widgets->radio_scal2, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal4, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal6, TRUE);
    gtk_widget_set_sensitive(widgets->radio_scal8, TRUE);
    switch (widgets->zboystruct->workconf->GraphicScaleFactor) {
      case 2:
      case 4:
      case 6:
      case 8: /* these factors are okay, leave them as is */
        break;
      case 5:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal4), TRUE);
        break;
      case 7:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal6), TRUE);
        break;
      default:
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets->radio_scal2), TRUE);
        break;
    }
    gtk_widget_set_sensitive(widgets->radio_scal1, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal3, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal5, FALSE);
    gtk_widget_set_sensitive(widgets->radio_scal7, FALSE);
  }
}

void radio_select_scal1(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 1;
}

void radio_select_scal2(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 2;
}

void radio_select_scal3(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 3;
}

void radio_select_scal4(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 4;
}

void radio_select_scal5(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 5;
}

void radio_select_scal6(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 6;
}

void radio_select_scal7(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 7;
}

void radio_select_scal8(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) == TRUE) widgets->zboystruct->workconf->GraphicScaleFactor = 8;
}

void spin_refreshlimit_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->fpslimit = gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
}

void combo_palette_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->palette = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void videosettingsscreen(GtkWidget *widget, gpointer userdata) {
  static struct zboygui_video widgets;
  GtkWidget *window, *vbox, *hbox_algos, *hbox_scaling, *hbox_refreshlimit, *hbox_combopal, *hbox_buttons, *label_title, *label_algos, *label_scaling, *label_refreshlimit, *label_refreshlimit_description, *label_empty0, *label_empty1, *label_empty2, *label_combopal, *alignement, *alignment_buttons, *chbox_colorization;
  GtkWidget *button_ok, *button_cancel;
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  widgets.zboystruct = (struct zboygtkgui_infos *)userdata;
  gtk_window_set_title(GTK_WINDOW(window), "zBoy video settings");
  gtk_window_set_default_size(GTK_WINDOW(window), 300, 150);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  alignement = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(alignement), 10, 10, 10, 10);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE); /* don't allow to manipulate the main window when this one is displayed */
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(widgets.zboystruct->parentwindow)); /* Don't allow to steal the focus of this window */
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_container_add(GTK_CONTAINER(window), alignement);
  gtk_container_add(GTK_CONTAINER(alignement), vbox);
  label_title = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label_title), "Video settings");
  gtk_label_set_justify(GTK_LABEL(label_title), GTK_JUSTIFY_CENTER);
  label_algos = gtk_label_new("Scaling algorithm:");
  label_scaling = gtk_label_new("Scaling ratio:");
  label_refreshlimit = gtk_label_new("Refresh rate [hz]:");
  label_combopal = gtk_label_new("Color palette:");
  label_refreshlimit_description = gtk_label_new("(use 60hz for best experience)");
  label_empty0 = gtk_label_new(NULL);
  label_empty1 = gtk_label_new(NULL);
  label_empty2 = gtk_label_new(NULL);
  widgets.spin_refreshlimit = gtk_spin_button_new_with_range(1, 60, 1);

  PopulateWorkConfFromCurConf(widgets.zboystruct); /* overwrite workconf with curconf */

  /* Prepare checkboxes */
  widgets.radio_alg_dummy = gtk_radio_button_new_from_widget(NULL);
  widgets.radio_alg_none = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_alg_dummy), "None");
  widgets.radio_alg_basic = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_alg_none), "Basic");
  widgets.radio_alg_2x = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_alg_none), "Scale2x");
  widgets.radio_alg_3x = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_alg_none), "Scale3x");
  widgets.radio_alg_eagle = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_alg_none), "Eagle");
  widgets.radio_scal1 = gtk_radio_button_new_with_label(NULL, "1x");
  widgets.radio_scal2 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "2x");
  widgets.radio_scal3 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "3x");
  widgets.radio_scal4 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "4x");
  widgets.radio_scal5 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "5x");
  widgets.radio_scal6 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "6x");
  widgets.radio_scal7 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "7x");
  widgets.radio_scal8 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(widgets.radio_scal1), "8x");
  chbox_colorization = gtk_check_button_new_with_label("Colorization (works only on a few ROMs)");

  /* Set the colorization checkbox accordingly to current settings */
  if (widgets.zboystruct->curconf->colorization != 0) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chbox_colorization), TRUE);

  /* Create the palette combobox */
  widgets.combo_palette = gtk_combo_box_text_new();
  gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.combo_palette), "gbmicro");
  gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(widgets.combo_palette), "peasoup");
  /* Set the current palette as active */
  gtk_combo_box_set_active(GTK_COMBO_BOX(widgets.combo_palette), widgets.zboystruct->workconf->palette);

  /* wire buttons */
  g_signal_connect(widgets.radio_alg_none, "toggled", G_CALLBACK(radio_select_algo_none), &widgets);
  g_signal_connect(widgets.radio_alg_basic, "toggled", G_CALLBACK(radio_select_algo_basic), &widgets);
  g_signal_connect(widgets.radio_alg_2x, "toggled", G_CALLBACK(radio_select_algo_2x), &widgets);
  g_signal_connect(widgets.radio_alg_3x, "toggled", G_CALLBACK(radio_select_algo_3x), &widgets);
  g_signal_connect(widgets.radio_alg_eagle, "toggled", G_CALLBACK(radio_select_algo_eagle), &widgets);
  g_signal_connect(widgets.radio_scal1, "toggled", G_CALLBACK(radio_select_scal1), &widgets);
  g_signal_connect(widgets.radio_scal2, "toggled", G_CALLBACK(radio_select_scal2), &widgets);
  g_signal_connect(widgets.radio_scal3, "toggled", G_CALLBACK(radio_select_scal3), &widgets);
  g_signal_connect(widgets.radio_scal4, "toggled", G_CALLBACK(radio_select_scal4), &widgets);
  g_signal_connect(widgets.radio_scal5, "toggled", G_CALLBACK(radio_select_scal5), &widgets);
  g_signal_connect(widgets.radio_scal6, "toggled", G_CALLBACK(radio_select_scal6), &widgets);
  g_signal_connect(widgets.radio_scal7, "toggled", G_CALLBACK(radio_select_scal7), &widgets);
  g_signal_connect(widgets.radio_scal8, "toggled", G_CALLBACK(radio_select_scal8), &widgets);
  g_signal_connect(widgets.spin_refreshlimit, "value-changed", G_CALLBACK(spin_refreshlimit_update), &widgets);
  g_signal_connect(widgets.combo_palette, "changed", G_CALLBACK(combo_palette_update), &widgets);
  g_signal_connect(chbox_colorization, "toggled", G_CALLBACK(togglebutton_uint8), &widgets.zboystruct->workconf->colorization);

  /* Set checkboxes accordingly to current params, and synch workparams with curparams */
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(widgets.spin_refreshlimit), widgets.zboystruct->workconf->fpslimit);
  switch (widgets.zboystruct->workconf->GraphicScaleFactor) {
    case 1:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal1), TRUE);
      break;
    case 2:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal2), TRUE);
      break;
    case 3:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal3), TRUE);
      break;
    case 4:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal4), TRUE);
      break;
    case 5:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal5), TRUE);
      break;
    case 6:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal6), TRUE);
      break;
    case 7:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal7), TRUE);
      break;
    case 8:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_scal8), TRUE);
      break;
  }
  switch (widgets.zboystruct->workconf->scalingalgo) {
    case REFRESHSCREEN_NOSCALE:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_alg_none), TRUE);
      break;
    case REFRESHSCREEN_BASICSCALE:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_alg_basic), TRUE);
      break;
    case REFRESHSCREEN_2X:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_alg_2x), TRUE);
      break;
    case REFRESHSCREEN_3X:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_alg_3x), TRUE);
      break;
    case REFRESHSCREEN_EAGLE:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widgets.radio_alg_eagle), TRUE);
      break;
  }

  /* Prepare the algos hbox */
  hbox_algos = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox_algos), label_algos, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_algos), widgets.radio_alg_none, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_algos), widgets.radio_alg_basic, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_algos), widgets.radio_alg_2x, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_algos), widgets.radio_alg_3x, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_algos), widgets.radio_alg_eagle, FALSE, FALSE, 0);

  /* Prepare the scaling hbox */
  hbox_scaling = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), label_scaling, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal1, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal2, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal3, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal4, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal5, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal6, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal7, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_scaling), widgets.radio_scal8, FALSE, FALSE, 0);

  /* Prepare the refresh limit input */
  hbox_refreshlimit = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox_refreshlimit), label_refreshlimit, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_refreshlimit), widgets.spin_refreshlimit, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_refreshlimit), label_refreshlimit_description, FALSE, FALSE, 0);

  /* Prepare the combo palette */
  hbox_combopal = gtk_hbox_new(FALSE, 10);
  gtk_box_pack_start(GTK_BOX(hbox_combopal), label_combopal, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_combopal), widgets.combo_palette, FALSE, FALSE, 0);

  /* pack the vbox with stuff */
  gtk_box_pack_start(GTK_BOX(vbox), label_title, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), label_empty0, FALSE, FALSE, 0); /* pack an empty space */
  gtk_box_pack_start(GTK_BOX(vbox), hbox_algos, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_scaling, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), label_empty1, FALSE, FALSE, 0); /* pack an empty space */
  gtk_box_pack_start(GTK_BOX(vbox), hbox_refreshlimit, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox_combopal, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), chbox_colorization, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), label_empty2, FALSE, FALSE, 0); /* pack an empty space */

  /* finally pack in OK / CANCEL buttons */
  hbox_buttons = gtk_hbox_new(FALSE, 10);
  alignment_buttons = gtk_alignment_new(1, 1, 0, 0);
  gtk_container_add(GTK_CONTAINER(alignment_buttons), hbox_buttons);
  button_ok = gtk_button_new_with_label("OK");
  button_cancel = gtk_button_new_with_label("Cancel");
  gtk_box_pack_start(GTK_BOX(hbox_buttons), button_cancel, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_buttons), button_ok, FALSE, FALSE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), alignment_buttons, FALSE, FALSE, 0);

  widgets.zboystruct->curwindow = window;
  g_signal_connect(G_OBJECT(button_ok), "clicked", G_CALLBACK(saveguiconfig), widgets.zboystruct);
  g_signal_connect(G_OBJECT(button_cancel), "clicked", G_CALLBACK(abortwindow), window);

  gtk_widget_show_all(window);
}


void fillup_inputconf_combo(GtkWidget *combo) {
  char *strptr, x;
  for (x = 0; (strptr = int2keystring(x)) != NULL; x++) gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(combo), strptr);
}

void combo_key_up_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_up = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_down_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_down = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_left_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_left = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_right_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_right = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_start_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_start = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_select_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_select = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_a_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_a = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_b_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_b = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_turboa_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_turboa = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_turbob_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_turbob = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F1_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_bckg = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F2_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_sprt = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F3_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_wind = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F5_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_save = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F7_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_load = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F8_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_asht = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F9_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_shot = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_F10_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_rset = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void combo_key_ESC_update(GtkWidget *widget, gpointer userdata) {
  struct zboygui_video *widgets = (struct zboygui_video *)userdata;
  widgets->zboystruct->workconf->key_quit = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
}

void inputscreen(GtkWidget *widget, gpointer userdata) {
  static struct zboygui_video widgets;
  GtkWidget *window, *vbox, *hbox_buttons, *label_title, *label_empty0, *label_empty1, *alignement, *alignment_buttons;
  GtkWidget *table;
  GtkWidget *label_up, *label_down, *label_left, *label_right, *label_start, *label_select, *label_a, *label_b, *label_turboa, *label_turbob;
  GtkWidget *label_F1, *label_F2, *label_F3, *label_F5, *label_F7, *label_F8, *label_F9, *label_F10, *label_ESC;
  GtkWidget *button_ok, *button_cancel;
  GtkWidget *combo_up, *combo_down, *combo_left, *combo_right, *combo_start, *combo_select, *combo_a, *combo_b, *combo_turboa, *combo_turbob;
  GtkWidget *combo_F1, *combo_F2, *combo_F3, *combo_F5, *combo_F7, *combo_F8, *combo_F9, *combo_F10, *combo_ESC;
  GtkWidget *separator;
  #if GTK_MAJOR_VERSION >= 3 /* since version 3 we should use gtk_separator_new() */
    separator = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
  #else
    separator = gtk_vseparator_new();
  #endif
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  widgets.zboystruct = (struct zboygtkgui_infos *)userdata;
  PopulateWorkConfFromCurConf(widgets.zboystruct); /* overwrite workconf with curconf */
  gtk_window_set_title(GTK_WINDOW(window), "zBoy input settings");
  gtk_window_set_default_size(GTK_WINDOW(window), 400, 150);
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  alignement = gtk_alignment_new(0.5, 0.5, 1, 1);
  gtk_alignment_set_padding(GTK_ALIGNMENT(alignement), 10, 10, 10, 10);
  gtk_window_set_modal(GTK_WINDOW(window), TRUE); /* don't allow to manipulate the main window when this one is displayed */
  gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(widgets.zboystruct->parentwindow)); /* Don't allow to steal the focus of this window */
  vbox = gtk_vbox_new(FALSE, 2);
  gtk_container_add(GTK_CONTAINER(window), alignement);
  gtk_container_add(GTK_CONTAINER(alignement), vbox);
  table = gtk_table_new(5, 10, FALSE);
  gtk_table_set_row_spacings(GTK_TABLE(table), 1);
  gtk_table_set_col_spacings(GTK_TABLE(table), 8);
  label_title = gtk_label_new(NULL);
  label_up = gtk_label_new("Up");
  label_down = gtk_label_new("Down");
  label_left = gtk_label_new("Left");
  label_right = gtk_label_new("Right");
  label_start = gtk_label_new("Start");
  label_select = gtk_label_new("Select");
  label_a = gtk_label_new("Button A");
  label_b = gtk_label_new("Button B");
  label_turboa = gtk_label_new("Turbo A");
  label_turbob = gtk_label_new("Turbo B");
  label_F1 = gtk_label_new("Bg switch");
  label_F2 = gtk_label_new("Spr switch");
  label_F3 = gtk_label_new("Win switch");
  label_F5 = gtk_label_new("Save");
  label_F7 = gtk_label_new("Load");
  label_F8 = gtk_label_new("Autoshot");
  label_F9 = gtk_label_new("Snapshot");
  label_F10 = gtk_label_new("Reset CPU");
  label_ESC = gtk_label_new("Quit");
  gtk_label_set_markup(GTK_LABEL(label_title), "Input configuration");
  gtk_label_set_justify(GTK_LABEL(label_title), GTK_JUSTIFY_CENTER);

  label_empty0 = gtk_label_new(NULL);
  label_empty1 = gtk_label_new(NULL);

  gtk_box_pack_start(GTK_BOX(vbox), label_title, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), label_empty0, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);

  /* Fill up every combobox and set it to the curconf position */
  combo_up = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_up);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_up), widgets.zboystruct->workconf->key_up);
  combo_down = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_down);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_down), widgets.zboystruct->workconf->key_down);
  combo_left = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_left);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_left), widgets.zboystruct->workconf->key_left);
  combo_right = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_right);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_right), widgets.zboystruct->workconf->key_right);
  combo_start = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_start);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_start), widgets.zboystruct->workconf->key_start);
  combo_select = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_select);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_select), widgets.zboystruct->workconf->key_select);
  combo_a = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_a);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_a), widgets.zboystruct->workconf->key_a);
  combo_b = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_b);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_b), widgets.zboystruct->workconf->key_b);
  combo_turboa = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_turboa);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_turboa), widgets.zboystruct->workconf->key_turboa);
  combo_turbob = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_turbob);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_turbob), widgets.zboystruct->workconf->key_turbob);
  combo_F1 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F1);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F1), widgets.zboystruct->workconf->key_bckg);
  combo_F2 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F2);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F2), widgets.zboystruct->workconf->key_sprt);
  combo_F3 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F3);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F3), widgets.zboystruct->workconf->key_wind);
  combo_F5 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F5);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F5), widgets.zboystruct->workconf->key_save);
  combo_F7 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F7);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F7), widgets.zboystruct->workconf->key_load);
  combo_F8 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F8);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F8), widgets.zboystruct->workconf->key_asht);
  combo_F9 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F9);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F9), widgets.zboystruct->workconf->key_shot);
  combo_F10 = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_F10);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_F10), widgets.zboystruct->workconf->key_rset);
  combo_ESC = gtk_combo_box_text_new();
  fillup_inputconf_combo(combo_ESC);
  gtk_combo_box_set_active(GTK_COMBO_BOX(combo_ESC), widgets.zboystruct->workconf->key_quit);

  /* wire every combobox to its callback */
  g_signal_connect(combo_up, "changed", G_CALLBACK(combo_key_up_update), &widgets);
  g_signal_connect(combo_down, "changed", G_CALLBACK(combo_key_down_update), &widgets);
  g_signal_connect(combo_left, "changed", G_CALLBACK(combo_key_left_update), &widgets);
  g_signal_connect(combo_right, "changed", G_CALLBACK(combo_key_right_update), &widgets);
  g_signal_connect(combo_start, "changed", G_CALLBACK(combo_key_start_update), &widgets);
  g_signal_connect(combo_select, "changed", G_CALLBACK(combo_key_select_update), &widgets);
  g_signal_connect(combo_a, "changed", G_CALLBACK(combo_key_a_update), &widgets);
  g_signal_connect(combo_b, "changed", G_CALLBACK(combo_key_b_update), &widgets);
  g_signal_connect(combo_turboa, "changed", G_CALLBACK(combo_key_turboa_update), &widgets);
  g_signal_connect(combo_turbob, "changed", G_CALLBACK(combo_key_turbob_update), &widgets);
  g_signal_connect(combo_F1, "changed", G_CALLBACK(combo_key_F1_update), &widgets);
  g_signal_connect(combo_F2, "changed", G_CALLBACK(combo_key_F2_update), &widgets);
  g_signal_connect(combo_F3, "changed", G_CALLBACK(combo_key_F3_update), &widgets);
  g_signal_connect(combo_F5, "changed", G_CALLBACK(combo_key_F5_update), &widgets);
  g_signal_connect(combo_F7, "changed", G_CALLBACK(combo_key_F7_update), &widgets);
  g_signal_connect(combo_F8, "changed", G_CALLBACK(combo_key_F8_update), &widgets);
  g_signal_connect(combo_F9, "changed", G_CALLBACK(combo_key_F9_update), &widgets);
  g_signal_connect(combo_F10, "changed", G_CALLBACK(combo_key_F10_update), &widgets);
  g_signal_connect(combo_ESC, "changed", G_CALLBACK(combo_key_ESC_update), &widgets);

  gtk_table_attach_defaults(GTK_TABLE(table), label_up, 0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), label_down, 0, 1, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(table), label_left, 0, 1, 3, 4);
  gtk_table_attach_defaults(GTK_TABLE(table), label_right, 0, 1, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), label_start, 0, 1, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), label_select, 0, 1, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), label_a, 0, 1, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), label_b, 0, 1, 8, 9);
  gtk_table_attach_defaults(GTK_TABLE(table), label_turboa, 0, 1, 9, 10);
  gtk_table_attach_defaults(GTK_TABLE(table), label_turbob, 0, 1, 10, 11);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_up, 1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_down, 1, 2, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_left, 1, 2, 3, 4);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_right, 1, 2, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_start, 1, 2, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_select, 1, 2, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_a, 1, 2, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_b, 1, 2, 8, 9);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_turboa, 1, 2, 9, 10);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_turbob, 1, 2, 10, 11);
  gtk_table_attach_defaults(GTK_TABLE(table), separator, 2, 3, 0, 11);     /* Separator */
  gtk_table_attach_defaults(GTK_TABLE(table), label_F1, 3, 4, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F2, 3, 4, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F3, 3, 4, 3, 4);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F5, 3, 4, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F7, 3, 4, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F8, 3, 4, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F9, 3, 4, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), label_F10, 3, 4, 8, 9);
  gtk_table_attach_defaults(GTK_TABLE(table), label_ESC, 3, 4, 9, 10);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F1, 4, 5, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F2, 4, 5, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F3, 4, 5, 3, 4);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F5, 4, 5, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F7, 4, 5, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F8, 4, 5, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F9, 4, 5, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_F10, 4, 5, 8, 9);
  gtk_table_attach_defaults(GTK_TABLE(table), combo_ESC, 4, 5, 9, 10);

  gtk_box_pack_start(GTK_BOX(vbox), label_empty1, FALSE, FALSE, 0);
  /* finally pack in OK / CANCEL buttons */
  hbox_buttons = gtk_hbox_new(FALSE, 10);
  alignment_buttons = gtk_alignment_new(1, 1, 0, 0);
  gtk_container_add(GTK_CONTAINER(alignment_buttons), hbox_buttons);
  button_ok = gtk_button_new_with_label("OK");
  button_cancel = gtk_button_new_with_label("Cancel");
  gtk_box_pack_start(GTK_BOX(hbox_buttons), button_cancel, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox_buttons), button_ok, FALSE, FALSE, 0);
  gtk_box_pack_end(GTK_BOX(vbox), alignment_buttons, FALSE, FALSE, 0);
  widgets.zboystruct->curwindow = window;
  g_signal_connect(G_OBJECT(button_ok), "clicked", G_CALLBACK(saveguiconfig), widgets.zboystruct);
  g_signal_connect(G_OBJECT(button_cancel), "clicked", G_CALLBACK(abortwindow), window);

  gtk_widget_show_all(window);
}


void zboygui_exit(GtkWidget *widget, gpointer userdata) {
  struct zboygtkgui_infos *zboygtkinfos = (struct zboygtkgui_infos *)userdata;
  zboygtkinfos->exitclicked = 1;
  gtk_main_quit();
}


void zboygui_play(GtkWidget *widget, gpointer userdata) {
  struct zboygtkgui_infos *zboygtkinfos = (struct zboygtkgui_infos *)userdata;
  zboygtkinfos->playclicked = 1;
  gtk_main_quit();
}


int zboygui(struct RomInformations *rominfo, struct zboyparamstype *emuconf) {
  static GtkWidget *window;
  static GtkWidget *label;
  static GtkWidget *vbox;
  static GtkWidget *alignment, *playbutton_alignment;
  static GtkWidget *menubar;
  static GtkWidget *filemenu, *settingsmenu, *helpmenu/*, *recentmenu*/;
  static GtkWidget *file,/* *recent,*/ *settings, *help;
  static GtkWidget *loadrom, *quit;
  static GtkWidget *about;
  static GtkWidget *generalsettings, *videosettings, *inputsettings;
  static GtkWidget *playbutton;
  char tempstring[1024], *pVer;
  static int gtkinitedalready = 0;

  static struct zboygtkgui_infos zboygtkinfos;
  struct zboyparamstype guiworkconf;
  zboygtkinfos.rominfo = rominfo;
  zboygtkinfos.workconf = &guiworkconf;
  zboygtkinfos.curconf = emuconf;

  zboygtkinfos.playclicked = 0;
  zboygtkinfos.exitclicked = 0;
  zboygtkinfos.newromtoload = 0;

  if (gtkinitedalready == 0) {
    gtkinitedalready = 1;
    gtk_init(NULL, NULL);

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    zboygtkinfos.parentwindow = window;
    pVer = zboyversion();
    snprintf(tempstring, 1024, "zBoy v%s", pVer);
    gtk_window_set_title(GTK_WINDOW(window), tempstring);
    gtk_window_set_default_size(GTK_WINDOW(window), 320, 240);
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);

    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(window), vbox);

    alignment = gtk_alignment_new(0.5, 1, 1, 1);
    gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 10, 10, 10, 10);
    
    playbutton_alignment = gtk_alignment_new(0.5, 1, 0, 0);
    gtk_alignment_set_padding(GTK_ALIGNMENT(playbutton_alignment), 10, 10, 10, 10);
    playbutton = gtk_button_new_with_label("Play!");
    gtk_container_add(GTK_CONTAINER(playbutton_alignment), playbutton);
    gtk_widget_set_sensitive(playbutton, FALSE); /* Gray out the Play menu (user needs to load a ROM first) */

    /* Create and populate the menu bar */
    menubar = gtk_menu_bar_new(); /* the whole menubar structure */
    filemenu = gtk_menu_new();    /* "File" menu */
    file = gtk_menu_item_new_with_label("File");
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(file), filemenu);
    loadrom = gtk_menu_item_new_with_label("Load a ROM");
    quit = gtk_menu_item_new_with_label("Quit");
    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), file);
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), loadrom);
    /* add and populate the recent submenu */
    /* recentmenu = gtk_menu_new();  */  /* "Recent" menu */
    /*recent = gtk_menu_item_new_with_label("Recent");
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(recent), recentmenu);
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), recent); */
    /* add last entry of the file menu (quit) */
    gtk_menu_shell_append(GTK_MENU_SHELL(filemenu), quit);

    settingsmenu = gtk_menu_new();    /* "Settings" menu */
    settings = gtk_menu_item_new_with_label("Settings");
    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), settings);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(settings), settingsmenu);
    generalsettings = gtk_menu_item_new_with_label("General");
    gtk_menu_shell_append(GTK_MENU_SHELL(settingsmenu), generalsettings);
    videosettings = gtk_menu_item_new_with_label("Video");
    gtk_menu_shell_append(GTK_MENU_SHELL(settingsmenu), videosettings);
    inputsettings = gtk_menu_item_new_with_label("Input");
    gtk_menu_shell_append(GTK_MENU_SHELL(settingsmenu), inputsettings);

    helpmenu = gtk_menu_new();    /* "Help" menu */
    help = gtk_menu_item_new_with_label("Help");
    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), help);
    gtk_menu_item_set_submenu(GTK_MENU_ITEM(help), helpmenu);
    about = gtk_menu_item_new_with_label("About");
    gtk_menu_shell_append(GTK_MENU_SHELL(helpmenu), about);

    /* Attach the menubar and the alignment container to the vbox */
    gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), playbutton_alignment, TRUE, FALSE, 0);

    /* Prepare the label */
    label = gtk_label_new(NULL);
    snprintf(tempstring, 1024, "\n\n\nzBoy GameBOY&#xAE; classic emulator\nCopyright &#xA9; Mateusz Viste %s\n\n\n\n", zboydate());
    gtk_label_set_markup(GTK_LABEL(label), tempstring);
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
    gtk_container_add(GTK_CONTAINER(alignment), label);

    /* Bind actions on menu's fiels */
    g_signal_connect(G_OBJECT(quit), "activate", G_CALLBACK(zboygui_exit), &zboygtkinfos);
    g_signal_connect(G_OBJECT(loadrom), "activate", G_CALLBACK(openfile), &zboygtkinfos);
    g_signal_connect(G_OBJECT(about), "activate", G_CALLBACK(aboutscreen), window);
    g_signal_connect(G_OBJECT(generalsettings), "activate", G_CALLBACK(generalsettingsscreen), &zboygtkinfos);
    g_signal_connect(G_OBJECT(videosettings), "activate", G_CALLBACK(videosettingsscreen), &zboygtkinfos);
    /* g_signal_connect(G_OBJECT(inputsettings), "activate", G_CALLBACK(NotImplementedYet), NULL); */
    g_signal_connect(G_OBJECT(inputsettings), "activate", G_CALLBACK(inputscreen), &zboygtkinfos);
    g_signal_connect(G_OBJECT(playbutton), "clicked", G_CALLBACK(zboygui_play), &zboygtkinfos);

    /* Register the gtk_main_quit routine to be called when the window is closed */
    g_signal_connect(G_OBJECT(window), "destroy", G_CALLBACK(zboygui_exit), &zboygtkinfos);

  }
  /* Display the window */
  gtk_widget_show_all(window);

  /* Start the GTK loop */
  do {
    gtk_main();
    if (zboygtkinfos.newromtoload != 0) {
      int x = LoadRom(rominfo);
      zboygtkinfos.newromtoload = 0;
      if (x == 0) {
          char tempstring[8192];
          char *NoYes[2] = {"No", "Yes"};
          gtk_widget_set_sensitive(playbutton, TRUE); /* ungray the play menu */
          snprintf(tempstring, 8192, "<span font_size=\"larger\">Loaded title: <span font_weight=\"bold\">%s</span></span>\n\nfile: %s\nlicensee: %s\ncart: %s\nROM: %d KiB\nRAM: %d KiB\nCRC: %8X\nbattery: %s\n", g_markup_escape_text(zboygtkinfos.rominfo->NiceTitle, strlen(zboygtkinfos.rominfo->NiceTitle)), g_markup_escape_text(zboygtkinfos.rominfo->ShortFilename, strlen(zboygtkinfos.rominfo->ShortFilename)), g_markup_escape_text(zboygtkinfos.rominfo->Licensee, strlen(zboygtkinfos.rominfo->Licensee)), g_markup_escape_text(zboygtkinfos.rominfo->CartTypeString, strlen(zboygtkinfos.rominfo->CartTypeString)), zboygtkinfos.rominfo->RomSize >> 10, zboygtkinfos.rominfo->RamSize >> 10, zboygtkinfos.rominfo->CrcSum, NoYes[zboygtkinfos.rominfo->Battery]);
          gtk_label_set_markup(GTK_LABEL(label), tempstring);
          gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
          gtk_alignment_set(GTK_ALIGNMENT(alignment), 0, 0, 0, 0); /* Don't make the alignment fill the whole vbox space anymore */
        } else {
          char *errstr = LoadRomErrStr(x);
          gtk_widget_set_sensitive(playbutton, FALSE); /* gray out the play menu */
          gtk_label_set_markup(GTK_LABEL(label), errstr);
      }
    }
  } while ((zboygtkinfos.exitclicked == 0) && (zboygtkinfos.playclicked == 0));

  if (zboygtkinfos.playclicked != 0) {
    gtk_widget_hide(window);
    while (gtk_events_pending ()) gtk_main_iteration(); /* pump out all pending GTK events */
  }

  if (zboygtkinfos.exitclicked != 0) return(1);

  return(0);
}
